// NullstellenBerechner.java                        MM 2013
import Tools.IO.*;                              // Eingaben

/**
 * Programm zur Nullstellenberechnung von Funktionen.
 * @author Monika Meiler
 */
public class NullstellenBerechner
{
/* ----------------------------------------------------- */
                                                // Attribut
/**
 * Speichert Differenzierbarkeit.
 */
  private boolean differenzierbar = true;

/* ----------------------------------------------------- */
                                                  // Dialog
/**
 * Nullstellenberechner:
 * Funktionseingabe, Nullstellenberechnung.
 */
  public void dialog()
  {
// Ueberschrift
    System.out.println();
    System.out.println( "Nullstellenberechner");

// Dialog
    char weiter = 'j';
    do
    {
// Neue Funktion
      Funktion fkt = funktionsAuswahl();
      if( fkt == null) continue;
      if( fkt.konsolenEingabe())         // Eingabe korrekt
      {

        System.out.println( "\nNullstelle von " +  fkt);

        do
        {
// Nullstellenberechnung
          System.out.println();
          berechneNullstelle( fkt);

// Weiter
          System.out.println();
          weiter = IOTools.readChar( "Neue Nullstelle (j/n)? ");
        } while( weiter == 'j');

      }
      weiter = IOTools.readChar( "Neue Funktion (j/n)? ");
    } while( weiter == 'j');

// Programm beendet
    System.out.println();
    System.out.println( "Programm beendet");
  }

/* ----------------------------------------------------- */
                                        // Funktionseingabe
/**
 * Funktionsauswahl.
 * @return Funktion
 */
  public Funktion funktionsAuswahl()
  {
// Menue aller Funktionen
    System.out.println( "Funktionsauswahl");
    System.out.println
    ( "  Gerade                      ... 1");
    System.out.println
    ( "  Polynom                     ... 2");
    System.out.println
    ( "  gebrochenrationale Funktion ... 3");

    int eingabe = IOTools.readInteger(": ");

// Festlegen der Funktion
    Funktion fkt = null;
    switch( eingabe)
    {
      case 1: fkt =  new Gerade();
              differenzierbar = true;
              break;
      case 2: fkt =  new Polynom( );
              differenzierbar = true;
              break;
      case 3: fkt = new RationaleFunktion();
              differenzierbar = false;
              break;
      default: System.out.println();
        System.out.println( "Fehlerhafte Eingabe!");
    }
    return fkt;
  }

/* ----------------------------------------------------- */
                                               // Iteration
/**
 * Verfahrensauswahl.
 * @param fkt differenzierbare Funktion
 */
  public void berechneNullstelle( Funktion fkt)
  {
// Berechnen der Nullstelle durch Newton
    if( differenzierbar)
    {
      DifferenzierbareFunktion dfkt
      = (DifferenzierbareFunktion)fkt;

      System.out.println
      ( " Gefundene Nullstelle: " + newton( dfkt));
    }
// Berechnen des Integrals durch Simpsonregel
    else
    {
      System.out.println
      ( " Gefundene Nullstelle: " + regulaFalsi( fkt));
    }
  }

/**
 * Berechnet Nullstelle nach Newton.
 * @param fkt DifferenzierbareFunktion
 * @return Nullstellennaeherung
  */
  private double newton( DifferenzierbareFunktion fkt)
  {
    System.out.println( "Newton:");
    System.out.println( "Eingabe der Iterationswerte");

    double x0 = IOTools.readDouble( " Startwert x0 = ");
    System.out.println( x0);

    double eps = IOTools.readDouble( " Genauigkeit eps = ");
    System.out.println( eps);

    int max;
    do
    {
      max = IOTools.readInteger
            ( " maximale Durchlaeufe > 0, max = ");
    } while( max < 1);
    System.out.println( max);

    Iteration it = new Iteration();
    return it.newton( fkt, x0, eps, max);
  }

/**
 * Berechnet Nullstelle nach Regula Falsi.
 * @param fkt Funktion
 * @return Nullstellennaeherung
 */
  private double regulaFalsi( Funktion fkt)
  {
    System.out.println( "Regula falsi:");
    System.out.println( "Eingabe der Iterationswerte");

    double x0 = 0, x1 = 0;
    do
    {
      x0 = IOTools.readDouble
           ( " 1. Naeherungswert, f(x0) > 0, x0 = ");
      System.out.println( x0);

      x1 = IOTools.readDouble
           ( " 2. Naeherungswert, f(x1) < 0, x1 = ");
      System.out.println( x1);
    } while( Math.signum( fkt.wert( x0))
              == Math.signum( fkt.wert( x1)));

    double eps = IOTools.readDouble( " Genauigkeit eps = ");
    System.out.println( eps);

    int max;
    do
    {
      max = IOTools.readInteger
           ( " maximale Durchlaeufe > 0, max = ");
    } while( max < 1);
    System.out.println( max);

    Iteration it = new Iteration();
    return  it.regulaFalsi( fkt, x0, x1, eps, max);
  }

/* ----------------------------------------------------- */
                                                // Programm
/**
 * Hauptprogramm, startet Nullstellenberechnung.
 */
  public static void main( String[] args)
  {
// Berechner erzeugen
    NullstellenBerechner berechner
    = new NullstellenBerechner();

// Berechner starten
    berechner.dialog();
  }
}

/* ----------------------------------------------------- */
                                               // Testwerte
/*
Nullstellenberechner
Nullstelle von 24.0 + -2.0 x^1 + -5.0 x^2 + 1.0 x^3

Newton:
 Startwert x0 = -3.0
 Genauigkeit eps = 1.0E-15
 maximale Durchlaeufe > 0, max = 100
 Gefundene Nullstelle: -2.0                           // -2

Newton:
 Startwert x0 = 0.0
 Genauigkeit eps = 1.0E-15
 maximale Durchlaeufe > 0, max = 100
 Gefundene Nullstelle: 3.9999999999999996              // 4

Newton:
 Startwert x0 = 1.0
 Genauigkeit eps = 1.0E-15
 maximale Durchlaeufe > 0, max = 100
 Gefundene Nullstelle: 3.0                             // 3

Nullstelle von
( -6.0 + 3.0 x^1 ) / ( 2.0 + -3.0 x^1 + 0.0 x^2 + 1.0 x^3 )

Regula falsi:
 1. Naeherungswert, f(x0) > 0, x0 = 1.5
 2. Naeherungswert, f(x1) < 0, x1 = 2.5
 Genauigkeit eps = 1.0E-15
 maximale Durchlaeufe > 0, max = 1000
 Gefundene Nullstelle: 2.0000000000000013              // 2


Neue Nullstelle (j/n)? 
Regula falsi:
Eingabe der Iterationswerte
 1. Naeherungswert, f(x0) > 0, x0 = -1.0
 2. Naeherungswert, f(x1) < 0, x1 = 2.5
 Genauigkeit eps = 1.0E-15
 maximale Durchlaeufe > 0, max = 1000
 Gefundene Nullstelle: 2.0                             // 2


Neue Nullstelle (j/n)? 
Regula falsi:
Eingabe der Iterationswerte
 1. Naeherungswert, f(x0) > 0, x0 = -2.5
 2. Naeherungswert, f(x1) < 0, x1 = 1.5
 Genauigkeit eps = 1.0E-15
 maximale Durchlaeufe > 0, max = 1000
 Gefundene Nullstelle: NaN
*/